import type { JCRNodeWrapper } from "org.jahia.services.content";
import type { RenderContext, Resource } from "org.jahia.services.render";
/** Initialize the registry with default url builders */
export declare function initUrlBuilder(): void;
/**
 * Generate a Jahia url for the provided node.
 *
 * @returns The final URL
 */
export declare function buildNodeUrl(
/** The node to build the URL for */
node: JCRNodeWrapper, config?: {
    /** The querystring parameters to append to the URL */
    parameters?: Record<string, string>;
    /**
     * The mode to use to build the URL. Defines the mode or override the one provided by the
     * renderContext.
     */
    mode?: "edit" | "preview" | "live";
    /**
     * The language to use to build the URL. Defines the languages or overrides the one provided by
     * the current resource
     */
    language?: string;
    /**
     * The extension to use to build the URL. Defines the extension or overrides the one provided by
     * the current resource
     */
    extension?: string;
}, context?: {
    /** Provided in react context, but you need to provide one otherwise. * */
    renderContext?: RenderContext;
    /** Provided in react context, you need to provide one otherwise. * */
    currentResource?: Resource;
}): string;
/**
 * Build a URL for a file in a module. Note that to be accessible, the folder that contains the file
 * must be part of the jahia.static-resources in package.json.
 *
 * The URL must not be absolute (start with a / or a protocol).
 */
export declare function buildModuleFileUrl(
/** Relative path of the file from the module's root (examples: css/my.css, images/myImage.webp) */
filePath: string, config?: {
    /** Defines a custom module name to access its files */
    moduleName?: string;
    /** Querystring parameters to append to the URL */
    parameters?: Record<string, string>;
}, context?: {
    /** Provided in react context, you need to provide one (or the module name) otherwise. */
    renderContext?: RenderContext;
}): string;
/** Build an url for any endpoint on the server (/cms/*, /graphql, ect). */
export declare function buildEndpointUrl(
/** Endpoint to call */
endpoint: string, config?: {
    /** Querystring parameters to append to the URL */
    parameters?: Record<string, string>;
}, context?: {
    /** Provided in react context, you need to provide one otherwise. */
    renderContext?: RenderContext;
}): string;
